package edu.sc.sasimulator;
import java.util.*;

/** 
 * @author Jose M. Vidal
 * @version $Revision: 1.3 $, $Date: 2000/04/25 19:23:14 $
 */
public class Agent extends Thing {

    public final int CLIMBSTEP = 1;

    Agent(Simulator sm, String n, double x, double y, Vector ss){
	super(sm, n, x, y);
	services = ss;
	myServices = new Vector();
	simulator.addEvent(this, (int)(Math.random()*(double)10.0), CLIMBSTEP);
    }

    /**Vector of all the Service agents out there */
    protected Vector services;
    /**
       * Get the value of services.
       * @return Value of services.
       */
    public Vector getServices() {return services;}
    
    /**
       * Set the value of services.
       * @param v  Value to assign to services.
       */
    public void setServices(Vector  v) {this.services = v;}

    /** A vector of all the services that I know are servicing me */
    protected Vector myServices;

    /** Adds a services to the list of services agents that
	service me.
	* @param s the service to add */
    public void addService(Service s){
	myServices.add(s);
    }

    /** Remove a service from the list of services agents that
	service me.
	* @param s the service */
    public void removeService(Service s){
	myServices.remove(s);
    }
    
    /**Handle event of type
       @param type the type of event
       @return true if I did something, false otherwise */
    public boolean doEvent(int type){
	switch (type) {
	case CLIMBSTEP:
	    return doClimbStep();
	default:
	    System.out.println("Cant figure out type=" + type);
	    return false;
	}
    }

    /**
     * @param a the allocation
     * @return my utility for allocation a */
    public double utility(Allocation a){
	Vector serversOfMine = a.getServices(this);
	double num = (double)serversOfMine.size();
	if (num <= 2)
	    return (num * 10);
	return (20 + 2*(num - 2));
    }

    /** Calculates the utility assuming that any sectors not listed
	are those for which we do not know who they are servicing(in anyone).
     * @param v vector of services who serve me
     * @param p the probability that a sensor for which we have no information
     is actually servicing me.
     * @return my utility given that myServices are serving me */
    public double expectedUtility(Vector v, double p){
	double num = (double)v.size() +
	    (double)(services.size() - v.size())*p;
	if (num <= 2)
	    return (num * 10);
	return (20 + 2*(num - 2));
    }
	

    public double utility(){
// 	System.out.println(name + "=" );
	double u = utility(simulator.getCurrentAllocation());
	return u;
    }

    /** Take one step up the hill. Thats, "up" from my point of view. Its greedy so
     it takes the biggest step possible, picking randomly if there is a tie.
    @return true if I did something.*/
    public boolean doClimbStep(){
	Allocation currentAllocation = simulator.getCurrentAllocation();
// 	System.out.println("CA before "+ name);
// 	System.out.print(currentAllocation.toString());

	Vector myServers = currentAllocation.getServices(this);

	//I know that I only increase utility by adding service, so who do I add?
	double maxUtility = -1;
	Vector bestServicesToAdd = new Vector();

	for (Enumeration i = services.elements(); i.hasMoreElements();){
	    Service is= (Service)i.nextElement();
	    if (myServers.contains(is))
		continue;
	    Allocation a = new Allocation(currentAllocation);
	    a.addServiceToAgent(is,this); 
	    double u = utility(a);        
	    if (is.isAgentOK(this) && (u > maxUtility)){
		maxUtility = u;
		bestServicesToAdd.removeAllElements();
		bestServicesToAdd.add(is);
	    }
	    else if (u == maxUtility)
		bestServicesToAdd.add(is);
	}
	simulator.addEvent(this, (int)(Math.random()*(double)10.0), CLIMBSTEP);
	Service bestOne = (Service)pickOne(bestServicesToAdd);
	if (bestOne == null)
	    return false;
// 	System.out.println("CA after");
// 	System.out.print(currentAllocation.toString());
// 	System.out.println("----");
	bestOne.addAgent(this); //actually add this service.
	return true;
    }

}
