package edu.sc.sasimulator;
import java.util.*;


/** An allocation of a number of Services to a number of Agents
    Each Service can be allocated (i.e. servicing) many agents and viz.
 * @author Jose M. Vidal
 * @version $Revision: 1.3 $, $Date: 2000/04/12 13:28:11 $
 */
public class Allocation {
    /** A vector of Service agents */
    public Vector services;

    /** A Vector of Vectors of Agents. The Service at services[i] is handling the
	agents at agents[i] */
    public Vector agents;

    public Allocation(){
	services = new Vector();
	agents = new Vector();
    }

    public Allocation(Vector s){
	services = (Vector)s.clone();
	agents = new Vector();
	for (Enumeration i = services.elements(); i.hasMoreElements();){
	    Service is = (Service)i.nextElement();
	    agents.add(new Vector());
	}
    }

    public Allocation(Allocation a){
	services = (Vector)a.services.clone();
	//	agents = (Vector)a.agents.clone();
	agents = new Vector();
	for (Enumeration i = a.agents.elements(); i.hasMoreElements();){
	    Vector iv = (Vector)i.nextElement();
	    agents.add(iv.clone());
	}
    }

    public String toString(){
	String res = new String();
	for (int i =0; i < services.size(); ++i){
	    Service is = (Service)services.elementAt(i);
	    res +=  is.getName() + ": ";
	    Vector v = (Vector)agents.elementAt(i);
	    for (int j=0; j < v.size(); ++j){
		Agent ja = (Agent)v.elementAt(j);
		res += ja.getName() + " ";
	    }
	    res += "\n";
	}
	return res;
    }

    /** Set the Service s to have an allocation of the agents given in a
     @param a must be a vector of Agent, or an empty vector (NOT null!)*/
    public void setServiceAllocation(Service s, Vector a){
	for (int i =0; i < services.size(); ++i){
	    Service is = (Service)services.elementAt(i);
	    if (is.equals(s)){
		agents.setElementAt(a,i);
		return;
	    }
	}
	//since it was not found, add it
	services.add(s);
	agents.add(a);
    }

    /** Makes s service a */
    public void addServiceToAgent(Service s, Agent a){
	Vector v = getAgents(s);
	if (v != null){
 	    if (!v.contains((Object)a)){
		v.add(a);
 	    }
	    return;
	}
	services.add(s);
	v = new Vector();
	v.add(a);
	agents.add(v);
    }

    /** Removes s from servicing a */
    public void removeServiceFromAgent(Service s, Agent a){
	Vector v = getAgents(s);
	if (v != null){
	    int size = v.size();
	    v.remove(a);
	}
	return;
    }

    /** Removes s from all the agents it services, so it services no one */
    public void removeServiceFromAll(Service s){
	if (s == null) return;
	Vector v = getAgents(s);
	if (v != null)
	    v.removeAllElements();
    }

    /** Return a vector of Agents which s services */
    public Vector getAgents(Service s){
	for (int i =0; i < services.size(); ++i){
	    Service is = (Service)services.elementAt(i);
	    if (is.equals(s)){
		Vector v = (Vector)agents.elementAt(i);
		return v;
	    }
	}
	return null;
    }

    /** Return a vector of Services which serve a */
    public Vector getServices(Agent a){
	Vector res = new Vector();
	for (int i =0; i < services.size(); ++i){
	    Service is = (Service)services.elementAt(i);
	    Vector v = (Vector)agents.elementAt(i);
	    if (v.contains(a))
		res.add(is);
	}
	return res;
    }

    public void merge(Allocation a){
	for (Enumeration i = a.services.elements(); i.hasMoreElements();){
	    Service is = (Service)i.nextElement();
	    services.add(is);
	}
	for (Enumeration i = a.agents.elements(); i.hasMoreElements();){
	    Vector iv = (Vector)i.nextElement();
	    agents.add(iv);
	}
    }
	
}
	
		
