package edu.sc.sasimulator;
import java.util.*;

/** 
 * @author Jose M. Vidal
 * @version $Revision: 1.4 $, $Date: 2000/04/25 19:23:14 $
 */
public class BiddingAgent extends Agent {

    /**The number of sensors I consider when calculating my marginal
       utility. The specific one are chosen at random on each iteration.
       This is K for the NK model.*/
    private int numNeighbors;
    /**
       * Get the value of numNeighbors.
       * @return Value of numNeighbors.
       */
    public int getNumNeighbors() {return numNeighbors;}
    
    /**
       * Set the value of numNeighbors.
       * @param v  Value to assign to numNeighbors.
       */
    public void setNumNeighbors(int  v) {this.numNeighbors = v;}
    

    /** The prior probability that a sensor is pointing towards me */
    private double probPoint;

    BiddingAgent(Simulator sm, String n, double x, double y, Vector ss){
	super(sm, n, x, y, ss);
	probPoint = 0.25;
    }


    /** Send my marginal utility to each of the services */
    public boolean doClimbStep(){
	Allocation currentAllocation = simulator.getCurrentAllocation();
	Vector servicesIKnowServiceMe = pickSome(myServices, numNeighbors);
	double currentUtility = expectedUtility(servicesIKnowServiceMe, probPoint); //.25 = 1/(#sectors+1)
	for (Enumeration i = services.elements(); i.hasMoreElements();){
	    BiddingService s= (BiddingService)i.nextElement();
	    double u;
	    if (servicesIKnowServiceMe.contains((Object)s)){
		Vector servicesWithoutS = (Vector)servicesIKnowServiceMe.clone();
		servicesWithoutS.removeElement(s);
		double oldutility = expectedUtility(servicesWithoutS, probPoint);
		u = currentUtility - oldutility;
	    }
	    else { //Currently I assume s is unknown
		Vector servicesWithS = (Vector)servicesIKnowServiceMe.clone();
		servicesWithS.add(s);
		double newutility = expectedUtility(servicesWithS, probPoint);
		u = newutility - currentUtility;
	    }
	    s.bid(this,u);

// 	for (Enumeration i = services.elements(); i.hasMoreElements();){
// 	    BiddingService is= (BiddingService)i.nextElement();
// 	    Allocation olda = new Allocation(currentAllocation);
// 	    Allocation newa = new Allocation(currentAllocation);
// 	    olda.removeServiceFromAgent(is,this);
// 	    newa.addServiceToAgent(is,this);
// 	    double u = utility(newa) - utility(olda);
// 	    is.bid(this,u);
// 	}

	}
	simulator.addEvent(this, (int)(Math.random()*(double)10.0), CLIMBSTEP);
// 	simulator.addEvent(this,10, CLIMBSTEP);
	return false; //I never change the utility.
    }

}
