package edu.sc.sasimulator;
import java.util.*;

/** 
 * @author Jose M. Vidal
 * @version $Revision: 1.5 $, $Date: 2000/04/25 19:23:14 $
 */
public class BiddingService extends Service {

    /** The agents that have bid (a Vector of BiddingAgent) */
    Vector agents;

    /** and their bids, a vector of Double*/
    Vector bids;

    public final int CLEAR = 1;

    BiddingService(Simulator sm, String n, double x, double y){
	super(sm, n, x, y);
	agents = new Vector();
	bids = new Vector();
	numAgentBids = 5;
	simulator.addEvent(this, (int)(Math.random()*(double)10.0), CLEAR);
    }

    public boolean doEvent(int type){
	switch (type) {
	case CLEAR:
	    return doClear();
	default:
	    System.out.println("Cant figure out type=" + type);
	    return false;
	}
    }

    /** The number of other agent bids I consider */
    private int numAgentBids;

    /**
       * Get the value of numAgentBids.
       * @return Value of numAgentBids.
       */
    public int getNumAgentBids() {return numAgentBids;}
    
    /**
       * Set the value of numAgentBids.
       * @param v  Value to assign to numAgentBids.
       */
    public void setNumAgentBids(int  v) {this.numAgentBids = v;}
    

    /** Go thru all the bids, pick the sector with the highest
    aggregate demand as long as that aggregate utility is greater
    than the utility I get for doing nothing.*/
    private boolean doClear(){
	double[] aggregateUtilities = new double[3]; //three sectors, 0,1,2

  	Vector agentsIConsider = pickSome(agents, numAgentBids);
//  	Vector agentsIConsider = agents;

	for (int i=0; i < agentsIConsider.size(); i++){
	    Agent a = (Agent)agentsIConsider.elementAt(i);
 	    int j = agentsIConsider.indexOf(a);
 	    double b = ((Double)bids.elementAt(j)).doubleValue();
//  	    double b = ((Double)bids.elementAt(i)).doubleValue();
	    int sector = determineSector(a);
	    aggregateUtilities[sector] += b;
	}
	double maxUtility = -1;
	int bestSector = -1;
	for (int i =0; i < 3; i++){
	    if (aggregateUtilities[i] > maxUtility){
		maxUtility = aggregateUtilities[i];
		bestSector = i;
	    }
	}
	if (!isOK(simulator.getCurrentAllocation()))
	    System.out.println("OOPS-1 ");
	
	if (maxUtility > utilityOfIdleness) { //if it pays me (and GU) to work, then...
	    for (int i=0; i < agents.size(); i++){
		Agent a = (Agent)agents.elementAt(i);
		int sector = determineSector(a);
		if (sector == bestSector) 
		    addAgent(a);
		if (!isOK(simulator.getCurrentAllocation()))
		    System.out.println("OOPS-2");
	    }
	}
	else { //Im better of doing nothing
	    removeAllAgents();
	}

	if (!isOK(simulator.getCurrentAllocation()))
	    System.out.println("OOPS");
	simulator.addEvent(this, (int)(Math.random()*(double)10.0), CLEAR);
	//	simulator.addEvent(this, 10, CLEAR);
	return true; //I always change allocation
    }


    /** Add bid to my list */
    public void bid(BiddingAgent a, double u){
	int index = agents.indexOf(a);
	Double d = new Double(u);
	if (index == -1) {
	    agents.add(a);
	    bids.add(d);
	    return;
	}
	agents.setElementAt(a,index);
	bids.setElementAt(d,index);
    }

}
