package edu.sc.sasimulator;
import java.awt.*;
import java.applet.Applet;
import java.util.*;
import java.io.*;

/** The main class.
 * @author Jose M. Vidal
 * @version $Revision: 1.5 $, $Date: 2000/04/25 19:23:14 $
 */
public class SASimulator extends Applet{
    static int numAgents;
    static int numServices;
    static String type;
    
    static Vector services;
    static Vector agents;
    
    static Simulator simulator;
    static int numSteps;
    static int numRuns;
    static int numTries;
    
    /** The max number of services the agent considers each time it selects a bid, chosen
	randomly */
    static int numNeighborsAgent;
  
    /** The max number of agent bids BiddingService agents consider when clearing, chosen
	randomly */
    static int numNeighborsService;
    double[] changes;
    double[] steps;
    
    /** Start everything */
    public void init() {
			services = new Vector();
			agents = new Vector();
			simulator = new Simulator(numSteps);
			changes = new double[numSteps];
	steps = new double[numSteps];
    }
    
    private void doRun(String type, PrintStream fileout,
		       double[] sx, double[] sy, double[] ax, double[] ay){
	double maxUtility  = -1;
	for (int trynum = 0 ; trynum < numTries; ++trynum){
	    System.out.println("Try=" + trynum);
	    
	    simulator.start();
	    if (type.equals("simple")){
		for (int i = 0; i < numServices; i++){
		    Service s = new Service(simulator, "S"+i, sx[i], sy[i]);
		    services.add(s);
		}
		for (int i = 0; i < numAgents; i++){
		    Agent a = new Agent(simulator, "A"+i, ax[i], ay[i], services);
		    agents.add(a);
		};
	    }
	    else if (type.equals("bidding")){
		for (int i = 0; i < numServices; i++){
		    BiddingService s = new BiddingService(simulator, "S"+i, sx[i], sy[i]);
		    s.setNumAgentBids(numNeighborsService);
		    services.add(s);
		}
		for (int i = 0; i < numAgents; i++){
		    BiddingAgent a = new BiddingAgent(simulator, "A"+i, ax[i], ay[i], services);
		    a.setNumNeighbors(numNeighborsAgent);
		    agents.add(a);
		};
	    }

	    simulator.setFileout(fileout);
	    simulator.setAgents(agents);
	    simulator.setServices(services);
	    if (maxUtility == -1) {
// 		System.out.println("Finding globally max utility");
		maxUtility = simulator.getMaxUtility();
		//System.out.println("maxUtility = " + maxUtility);
	    }
	    simulator.run();
	    fileout.println();

	    for (int i=numSteps - 1; i > 0; i--){
		//Count all changes
		int discretizedAlpha = (int)(50.0*simulator.utilities[i]);
		if (discretizedAlpha == 50)  //if alpha was 1, we count it in the .98--1 bin
		    discretizedAlpha = 49; 
		threeDPlot[i][discretizedAlpha]++;
	    }

// 	    System.out.println("max = " + maxUtility + " last=" + simulator.getCurrentUtility());
	    double alpha = simulator.getCurrentUtility() / maxUtility;
// 	    System.out.println("alpha = " + alpha);
	    agents.removeAllElements();
	    services.removeAllElements();
	}
    }
    
    
    private void printInfoToFile(PrintStream p){
	Date now = new Date();
	p.println("#" + now.toString());
	p.println("#numAgents = " + numAgents);
	p.println("#numServices = " + numServices);
	p.println("#numSteps = " + numSteps);
	p.println("#numTries = " + numTries);
	p.println("#numRuns = " + numRuns);
	p.println("#numNeighborsAgent = " + numNeighborsAgent);
	p.println("#numNeighborsService = " + numNeighborsService);
    }


    private int[][] threeDPlot;
  
    private void doRuns(){
	double[] sx = new double[numServices];
	double[] sy = new double[numServices];
	double[] ax = new double[numAgents];
	double[] ay = new double[numAgents];
    
	//Indeces are time, discretized alpha (50), and contains the number of runs+tries for which its true.
	threeDPlot = new int[numSteps][50];

	PrintStream fileout = null;
	PrintStream fileout2 = null;
	PrintStream difference = null;
	try{

	    if (type.equals("simple")){
		fileout = new PrintStream(new FileOutputStream("simple.dat"));
		printInfoToFile(fileout);
	    }
	    else if (type.equals("bidding")){
		fileout2 = new PrintStream(new FileOutputStream("bidding.dat"));
		printInfoToFile(fileout2);
	    }
	    else {
		fileout = new PrintStream(new FileOutputStream("simple.dat"));
		fileout2 = new PrintStream(new FileOutputStream("bidding.dat"));
		difference = new PrintStream(new FileOutputStream("difference.dat"));
	    }
	    double[] utils = new double[numSteps];

	    for (int run =0; run < numRuns; run++){
		System.out.println("run=" + run);

		//Pick locations for services and agents(targets)
		for (int i = 0; i < numServices; i++){
		    sx[i] = Math.random()*100.0;
		    sy[i] = Math.random()*100.0;
		}
		for (int i = 0; i < numAgents; i++){
		    ax[i] = Math.random()*100.0;
		    ay[i] = Math.random()*100.0;
		};

		if (type.equals("simple")){
		    doRun(type, fileout, sx, sy, ax, ay);
		}
		else if (type.equals("bidding")){
		    doRun(type, fileout2, sx, sy, ax, ay);
		}
		else {
		    doRun("simple", fileout, sx, sy, ax, ay);
		    for (int j=0; j< numSteps; j++){
			utils[j] = simulator.utilities[j];
		    }
		    doRun("bidding", fileout2, sx, sy, ax, ay);
		    for (int j=0; j< numSteps; j++){
			double diff = simulator.utilities[j] - utils[j];
			difference.println(j + "\t" + diff);
		    }
		    difference.println();
		}

	    }//end runs
	    String threeDfilename = new String("3dplot");
	    threeDfilename += "." + numAgents + "." + numServices + "." + numNeighborsAgent + "." +
		numNeighborsService + ".dat";
	    PrintStream threeDPlotFile = new PrintStream(new FileOutputStream(threeDfilename));
	    printInfoToFile(threeDPlotFile);
	    for (int i = 0; i < numSteps; i++){
		for (int a = 0; a < 50; a++) {
		    double ab = (double)a/50;
		    double percentage = (double)threeDPlot[i][a]*100.0/(double)(numRuns*numTries);
		    threeDPlotFile.println(i + "\t" +  ab + "\t" + percentage);
		}
		threeDPlotFile.println();
	    }
	    
	    if (fileout != null)
		fileout.close();
	    if (fileout2 != null)
		fileout2.close();
	}
	catch (IOException e) {
	    System.out.println("Could not write to file");
	};
    }

    public void start() {
	System.out.println("SASimulator start");
	try{
	    doRuns();
	    
	    PrintStream changefileout = new PrintStream(new FileOutputStream("change.dat"));
 	    PrintStream stepsfileout = new PrintStream(new FileOutputStream("steps.dat"));

	    for (int i=0; i < numSteps; i++){
		steps[i] /= changes[i];
		changes[i] /= (double)numRuns;
		String st = i + "\t" + steps[i];
		String ch = i + "\t" + changes[i];
		changefileout.println(ch);
		stepsfileout.println(st);
	    }
	    changefileout.close();
	    stepsfileout.close();
	}
	catch (IOException e) {
	System.out.println("Could not write to file");
	};
    }


    public static void main(String a[]){
	numAgents = 7;
	numServices = 7;
	numSteps = 100;
	numTries = 10;
	numRuns = 100;

	//It probably only makes sense for these two to be the same number
	numNeighborsAgent = 1;
	numNeighborsService = 1;


	type = "bidding";
	try{
	    for( int c = 0 ; c < a.length ; c += 2 )
		{
		    if( a[c].equals("-agents") || a[c].equals("-a") ){
			numAgents = Integer.parseInt(a[c+1]);
		    }
		    else if ( a[c].equals("-services") || a[c].equals("-s") ){
			numServices = Integer.parseInt(a[c+1]);
		    }
		    else if ( a[c].equals("-type") || a[c].equals("-t") ){
			type = a[c+1];
		    }
		    else if ( a[c].equals("-runs") || a[c].equals("-r") ){
			numRuns = Integer.parseInt(a[c+1]);
		    }
		    else if ( a[c].equals("-steps") || a[c].equals("-st") ){
			numSteps = Integer.parseInt(a[c+1]);
		    }
		    else {
			System.out.println("Did not understand: " + a[c]);
			throw new Exception();
		    }
		}
	    SASimulator sa = new SASimulator();
	    sa.init();
	    sa.start();
	}
	catch (Exception e) {
	    System.err.println(e.toString());
	    System.err.println("");
	    System.err.println("USAGE: SASimulator [-parameter value]");
	    System.err.println("");
	    System.err.println("    Parameters     value        default");
	    System.err.println("   ------------------------------------");
	    System.err.println("    -agents (-a)   num of agents       " + numAgents);
	    System.err.println("    -services (-s) num of services     " + numServices);
	    System.err.println("    -type (-t)     [simple|bidding] " + type);
	    System.err.println("    -runs (-r)     number of runs      " + numRuns);
	    System.err.println("    -steps (-st)   number of steps   " + numSteps);
	    e.printStackTrace();
	};
		    
    }
}
// -*- mode:JDE; tab-width:4; c-basic-offset:2; indent-tabs-mode:nil -*- 
