package edu.sc.sasimulator;
import java.util.*;

/** 
 * @author Jose M. Vidal
 * @version $Revision: 1.3 $, $Date: 2000/04/25 19:23:14 $
 */
public class Service extends Thing {

    Service(Simulator sm, String n, double x, double y){
	super(sm, n, x, y);
    }

    /** The utility I get for doing nothing (from the energy I save)*/
    protected double utilityOfIdleness = 5;

    /** I get a utility for not doing anything (yay!), but zero for actually doing something */
    public double utility(Allocation a){
	Vector agentsICover = a.getAgents(this);
	if ((agentsICover == null) || (agentsICover.size() == 0))
	    return utilityOfIdleness;
	return 0;
    }

    /** Is the allocation a OK with me? 
	@return true if a does not have me scanning more than 1 sector, false otherwise */
    public boolean isOK(Allocation a){
	Vector agentsICover = a.getAgents(this);
	if (agentsICover == null)
	    return true;
	int sector = -1;
	for (Enumeration i = agentsICover.elements(); i.hasMoreElements();){
	    Thing t = (Thing)i.nextElement();
	    int s = determineSector(t);
	    if (sector == -1)
		sector = s;
	    else if (s != sector){
//  		System.out.println(toString() + " sector=" + sector);
//  		System.out.println(t.toString() + " sector=" + s);
		return false;
	    }
	}
	return true;
    }

    /** Could I point towards a. Always returns true, but if we added a radius to the sensors, this
     could change.*/
    public boolean isAgentOK(Agent a){
	return true;
// 	return (20*20 > (a.getXLocation() - getXLocation())*(a.getXLocation() - getXLocation()) +
// 		(a.getYLocation() - getYLocation())*(a.getYLocation() - getYLocation()));
    }
	
    /** Determines which sector the sensor in  sx, sy needs to point to in order to 
	see something in tx,ty
     * @param l the location we want the sensor to point to. */
    public int determineSector(double sx, double sy, double tx,double ty){

	double angleToTarget = adjustedAtan(tx-sx, ty-sy);
	int activeSector = 0;;

	if (angleToTarget <= Math.toRadians(60) || angleToTarget > Math.toRadians(300))
	    activeSector = 0;
	else if (angleToTarget > Math.toRadians(60) && angleToTarget <= Math.toRadians(180))
	    activeSector = 1;
	else
	    activeSector = 2;

	return activeSector;
    }

    public int determineSector(double tx,double ty){
	return determineSector(xLocation, yLocation, tx, ty);
    }

    public int determineSector(Agent a){
	return determineSector(xLocation, yLocation, a.getXLocation(), a.getYLocation());
    }

    public int determineSector(Thing a){
	return determineSector(xLocation, yLocation, a.getXLocation(), a.getYLocation());
    }


    /** Calculate the angle given the delta-x and delta-y. I stole this one from Radsim
     * @param diff_X 
     * @param diff_y
     * @return the angle */
    public double adjustedAtan( double diff_X, double diff_Y ) {
        double goodAngle, stdAtan;
        stdAtan = Math.atan( diff_Y / diff_X );  // defined for result between -PI/2 and +PI/2
        if (    ( diff_Y >= 0 && diff_X <= 0 )          // 2nd quadrant
             || ( diff_Y <= 0 && diff_X <= 0 ) )        // 3rd quadrant
            goodAngle =  Math.PI  +  stdAtan;
        else if ( diff_Y < 0 && diff_X > 0 )          // 4th quadrant
            goodAngle = ( 2.0 * Math.PI ) + stdAtan;
        else goodAngle = stdAtan;                     // 1st quadrant
        return goodAngle;
    }

    /** Give service to a, update currentAllocation. Removes service to other agents if needed.*/
    public void addAgent(Agent a){
	//iterate over the copy
	Allocation currentAllocationCopy = new Allocation(simulator.getCurrentAllocation());

	//then delete from the original. If we do both on the original the enumeration gets confused
	Allocation currentAllocation = simulator.getCurrentAllocation();

	Vector agentsICover = currentAllocationCopy.getAgents(this);
	int sector = determineSector(a);
	boolean ok = true;


	if (!isOK(currentAllocation))
	    System.out.println("ERROR: in addAgent: 1");
	//remove all the ageents that need a different sector
	if (agentsICover != null){
	    for (Enumeration i = agentsICover.elements(); i.hasMoreElements();){
		Agent t = (Agent)i.nextElement();
		int s = determineSector(t);
		if (s != sector){
// 		    System.out.println("Removing " + t.toString() + " at sector " + s);
		    removeAgent(t);
//		    currentAllocation.removeServiceFromAgent(this,t);
		    ok = false;
		}
	    }
	}
	addMyServiceToAgent(a);
//   	currentAllocation.addServiceToAgent(this,a);
	if (!isOK(currentAllocation)){ //debugging check
	    System.out.println("ERROR: in addAgent");
	}
	return;
    }

    /** Remove it */
    public void removeAgent(Agent a){
	Allocation currentAllocation = simulator.getCurrentAllocation();
	currentAllocation.removeServiceFromAgent(this,a);
	a.removeService(this);
    }

    public void addMyServiceToAgent(Agent a){
	Allocation currentAllocation = simulator.getCurrentAllocation();
	currentAllocation.addServiceToAgent(this,a);
	a.addService(this);
    }

    /** Make this service idle */
    public void removeAllAgents(){
	Allocation currentAllocation = simulator.getCurrentAllocation();

	//remove myself from all those agents
	Vector agentsICover = currentAllocation.getAgents(this);
	if (agentsICover == null)
	    return;
	for (Enumeration i = agentsICover.elements(); i.hasMoreElements();){
	    Agent t = (Agent)i.nextElement();
	    t.removeService(this);
	}

	//remove me from the currentAllocation
	currentAllocation.removeServiceFromAll(this);
    }
    
}
